@@ -349,8 +349,9 @@ class PayUIDN(models.Model):
     @classmethod
     def signature(cls, payload, merchant_key):
         confirmation_hash = text_type().join(
-            [text_type("%s%s") % (len(text_type(payload[field])), payload[field])
-             for field in payload]
+            [text_type('{length}{value}').format(
+                    length=len(text_type(value).encode('utf-8')), value=value
+            ) for value in payload.values()]
         ).encode('utf-8')
         return hmac.new(merchant_key, confirmation_hash).hexdigest()
 
@@ -373,8 +374,12 @@ class PayUIDN(models.Model):
 class PayUToken(models.Model):
     ipn = models.OneToOneField(PayUIPN)
 
-    # same value as IPN's REFNO
-    IPN_CC_TOKEN = models.CharField(max_length=9, verbose_name="Token")
+    # used for token/v1 api payments (same value as IPN's REFNO)
+    # DEPRECATED
+    IPN_CC_TOKEN = models.CharField(max_length=9, verbose_name="Token (deprecated)", blank=True, null=True)
+
+    # can be used for the `IPN_CC_TOKEN` field in alu/v3 token payments
+    TOKEN_HASH = models.CharField(max_length=64, verbose_name="Token", blank=True, null=True)
 
     # documentation is unclear of the length and format of this field
     IPN_CC_MASK = models.CharField(max_length=36, verbose_name="Last 4 digits")
@@ -382,11 +387,11 @@ class PayUToken(models.Model):
     IPN_CC_EXP_DATE = models.DateField(verbose_name="Expiration date")
 
     class Meta:
-        verbose_name = 'PayU Tokens V1'
+        verbose_name = 'PayU Tokens'
         app_label = 'payu'
 
     def __unicode__(self):
-        return u'<Token: %s>' % self.IPN_CC_TOKEN
+        return u'<Token: %s>' % (self.TOKEN_HASH or self.IPN_CC_TOKEN)
 
 
 @receiver(post_save, sender=PayUToken)
